function run_robotics(datapath)
% usage: run_robotics(datapath)
%   Evaluates feature hallucination approach on the robotics dataset.
%
% Input:
%  * datapath: path where data file 'robotics_features' and split file
%  'robotics_splits' are stored.
%

% load features
load([datapath '/robotics_features'], 'D_webcam', 'D_dslr', 'L');

% load data splits
load([datapath '/robotics_splits'], 'splits', 'N');
n = length(splits);
S = length(splits{1});

% set parameters
d = 200;
weights = [0.5 0.5];
sigma = 1e-2;
T = 10;
B = 2;
conf_measure = 'margin';

% run feature hallucination
fprintf('Running feature hallucination...\n');
hallucinate_ccrs = zeros(S,n);
for ii = 1:n
  fprintf('N=%d\n', N(ii));
  hallucinate_ccrs(:,ii) = ...
      feature_hallucinate(D_webcam, D_dslr, L, d, splits{ii}, weights, sigma);
end
fprintf('done.\n\n');

% run with bootstrapping
fprintf('Running feature hallucination with bootstrapping...\n');
bootstrap_ccrs = zeros(S,n);
for ii = 1:n
  fprintf('N=%d\n', N(ii));
  bootstrap_ccrs(:,ii) = ...
      feature_hallucinate(D_webcam, D_dslr, L, d, ...
                          splits{ii}, weights, sigma, T, B, conf_measure);
end
fprintf('done.\n\n');

% plot results
figure; hold on;
errorbar(N, mean(hallucinate_ccrs), std(hallucinate_ccrs), 'r-', 'LineWidth', 2.0);
errorbar(N, mean(bootstrap_ccrs), std(bootstrap_ccrs), 'k-', 'LineWidth', 2.0);
grid on;
legend('Hallucination', 'Hallucination with Bootstrap', 'Location', 'SouthEast');
xlabel('Max. Number of Labeled Examples Per Class');
ylabel('Correct Classification Rate (CCR)');
title('Robotics Dataset');
