function [K, dK] = covexpsum(logtheta,x,sigma)
% usage: [K, dK] = covexpsum(logtheta,x,sigma)
%  Implements sum of exponential kernels.
%
% logtheta = [w_1,...,w_V,ell_1,...,ell_V];
%
% Returns K = sum_i w_i exp(-ell_i*x{i}) + sigma*I.
%
  
% get sigma  
if exist('sigma')~=1 || isempty(sigma)
  sigma = 0;
end

% properties
V = length(x);
p = length(logtheta);
if p~=2*V
  error('logtheta length is not 2*V');
end

% get hyper-parameters
V = length(logtheta)/2;
w = exp(logtheta(1:V));
ell = exp(logtheta((1:V)+V));

% compute kernel
K = 0;
for v = 1:V
  K = K + w(v)*exp(-ell(v)*x{v});
end
K = K + sigma*eye(size(K));

% TODO
if nargout==2
  dK = [];
end