function [CCR,pL,NLL,f,vars] = gpeval(logtheta,covfunc,x,L,S,U,sigma)
% usage: [CCR,pL,NLL,f,vars] = gpeval(logtheta,covfunc,x,L,S,U,sigma)
%  Evaluate GP model for classification (still assuming Gaussian noise).
%
% Input:
%  * logtheta: log parameter vector
%  * covfun: handle to covariance function
%  * x: input data (format dependent on covfun)
%  * L: vector of class labels
%  * S: labeled example indices
%  * U: test example indices
%  * sigma: Gaussian noise variance (default 0)
%
% Output:
%  * CCR: classification accuracy
%  * NLL: negative log likelihood of GP
%  * f: |U| x nC mean prediction matrix, nC=|unique(L)| is the number of
%       classes
%  * vars: |U| x nC variance prediction matrix
%  * pL: |U| x 1 predicted label vector
%

% get sigma
if exist('sigma')~=1 || isempty(sigma)
  sigma = 0;
end

% create one vs. all labels
[y,C,nC] = makeonevall(L);
if nC==2
  nC = 1;
end

if ischar(covfunc)
    eval(sprintf('covfunc=@%s;',covfunc));
end
K = covfunc(logtheta,x);

K_ = K(S,S) + sigma.*eye(length(S));
K_star = K(U,S);
Kss = diag(K); 
Kss_test = reshape(Kss(U),length(U),1);
alpha = K_star*inv(K_);
ntest = length(U);
f = zeros(ntest,nC);
vars = zeros(ntest,nC);
for ii = 1:nC
  f(:,ii) = alpha*y(S,ii);
  vars(:,ii) = Kss_test - sum(alpha.*K_star,2);
end

if nC>1
  [pL,mIds] = max(f,[],2);
  pL = C(mIds);
  CCR = ccr(pL,L(U));
else
  pL = f;
  pL(find(f<=0)) = -1;
  pL(find(f>0)) = 1;
  CCR = ccr(pL,y(U));
  
  % map pL back to original class labels
  pL(find(pL==1)) = min(C);
  pL(find(pL==-1)) = max(C);
end

NLL = gplik([],@coveye,K_,y(S,:));
