function [f,dfdx] = gplik(logtheta,covfun,x,y)
% usage: [f,dfdx] = gplik(logtheta,covfun,x,y)
%  Implements marginal likelihood of GP assuming Gaussian noise (GP
%  regression).
%

if nargout==2
  [K,dK] = covfun(logtheta,x);
else
  K = covfun(logtheta,x);
end

[L,p] = chol(K);
to_add = 1e-6;
%if p, fprintf('warning: singular matrix\n'); end;
while p
  K = K + to_add.*eye(size(K));
  [L,p] = chol(K);
  to_add = to_add * 10;
end
L = L';

[n,C] = size(y);
if C==2
  C = 1;
end

Kinv = L'\(L\eye(n));
Alpha = solve_chol(L',y);
logKdet = sum(log(diag(L)));
const = 0.5*n*log(2*pi);

f = 0.5*sum(sum(y.*Alpha)) + C*(logKdet + const);

if nargout==2
  P = length(logtheta);
  dfdx = zeros(size(logtheta));
  for c = 1:C
    W = Kinv - Alpha(:,c)*Alpha(:,c)';
    for p = 1:P
      dfdx(p) = dfdx(p) + sum(W(dK{p,2}).*dK{p,1})/2;
    end
  end
end
